//
// Project : Jam
// File    : ClassDeclaration.java
// Author  : Giovanni Lagorio (gio@libertyline.com)
//

package jam ;

import java.util.* ;
import invisible.jacc.parse.Token ;

class ClassDeclaration extends TypeDeclaration {

	private Modifiers modifiers ;
	private Name superclass ;
	private Vector interfaces ;

	ClassDeclaration(Modifiers modifiers_, String identifier_, Name superclass_, Vector interfaces_,
								Vector memberDecls_, Token sourcePosition_) {
		super(sourcePosition_, identifier_, memberDecls_) ;
		modifiers = modifiers_ ;
		superclass = superclass_ ;
		interfaces = interfaces_ ;
	}

	public void dump(TabPrintStream s) throws JamException {
		s.print(modifiers.toString()) ;
		if (superclass.toString().equals("java.lang.Object"))
			s.print("class "+identifier) ;
		else	s.print("class "+identifier+" extends "+superclass) ;
		if (interfaces.size()!=0) {
			s.print(" implements ") ;
			dumpNames(s, interfaces) ;
		}
		s.incLevel() ;
		s.println(" {\n") ;
		dumpDecls(s, memberDecls) ;
		s.decLevel() ;
		s.println("}\n") ;
	}

	void implResolveNames(CompilationUnit cu) throws JamException {
		superclass = cu.getFullyQualifiedName(superclass) ;
		interfaces = resolveNames(interfaces, cu) ;
		resolveDecls(memberDecls, cu) ;
	}

	private ClassType ct ;

	void createModuleObject(CompilationUnit cu) throws JamException {
		super.createModuleObject(cu) ;
		ct = new ClassType(fullName) ;
	}

	void extractModuleInformation(CompilationUnit cu) throws JamException {
		ct.init(superclass.toString(), interfaces, constructors(ct), fields(ct), methods(ct)) ;
	}

}

