//
// Project : Jam
// File    : CompilationUnit.java
// Author  : Giovanni Lagorio (gio@libertyline.com)
//

package jam ;

import java.util.* ;

class CompilationUnit {

	private Name packageName ;
	private Vector importDeclarations ;
	private Vector typeDeclarations ;
	private Vector declaredNames ; // all declared names; considering all input files
	private String filename ;

	CompilationUnit(Name packageName_, Vector importDeclarations_, Vector typeDeclarations_ ) {
		packageName = packageName_ ;
		importDeclarations = importDeclarations_ ;
		typeDeclarations = typeDeclarations_ ;
	}

	void setFilename(String filename_) {
		filename = filename_ ;
	}

	String getFilename() {
		return filename ;
	}

	Name getPackageName() {
		return packageName.copy() ;
	}

	void dump(TabPrintStream s) throws JamException {
		String name = packageName.toString() ;
		if ( name.length()!=0 )
			s.println("package "+name+" ;\n") ;
		//
		// Import declarations are useless: after processing all names are
		// qualified
		//
		//for(Enumeration e=importDeclarations.elements(); e.hasMoreElements();) {
		//	Decl d = (Decl)e.nextElement() ;
		//	d.dump(s) ;
		//}
		//s.println("") ;
		for(Enumeration e=typeDeclarations.elements(); e.hasMoreElements();)
			((Decl)e.nextElement()).dump(s) ;
	}

	private final static ImportDeclaration javaLang =
					new ImportDeclaration(new Name("java.lang"), true, null) ;

	Name getFullyQualifiedName(Name n) throws JamException {
		if (!n.isSimpleName()) {
			for(Enumeration e=declaredNames.elements() ; e.hasMoreElements() ;)
				if (n.equals(e.nextElement())) return n ;
			try	{
				Class.forName(n.toString()) ;
				return n ;
			} catch (ClassNotFoundException cnfe) {
				throw new JamException("Can't find type '"+n+"'.") ;
			}
		}
		String id = n.toString() ;
		Name pname = packageName.copy().appendComponent(id) ;
		for(Enumeration e=declaredNames.elements() ; e.hasMoreElements() ; )
			if (pname.equals(e.nextElement())) return pname ;
		for(Enumeration e=importDeclarations.elements(); e.hasMoreElements();) {
			ImportDeclaration impDecl = ((ImportDeclaration)e.nextElement()) ;
			pname = impDecl.getPackageName() ;
			if (pname!=null) {
				pname.appendComponent(id) ;
				for(Enumeration j=declaredNames.elements() ; j.hasMoreElements() ; )
					if (pname.equals(j.nextElement())) return pname ;
			}
		}
		Name matched = javaLang.match(id) ;
		if (matched!=null) return matched ;
		for(Enumeration e=importDeclarations.elements(); e.hasMoreElements();) {
			matched = ((ImportDeclaration)e.nextElement()).match(id) ;
			if (matched!=null) return matched ;
		}
		throw new JamException("Can't resolve type '"+n+"'.") ;
	}

	void resolveNames(Vector declaredNames_) throws JamException {
		declaredNames = declaredNames_ ;
		for(Enumeration e=typeDeclarations.elements(); e.hasMoreElements();)
			((Decl)e.nextElement()).resolveNames(this) ;
	}
	
	void createModuleObjects() throws JamException {
		for(Enumeration e=typeDeclarations.elements(); e.hasMoreElements();)
			((TypeDeclaration)e.nextElement()).createModuleObject(this) ;
	}

	void extractModuleInformation(boolean mixinDecls) throws JamException {
		Vector d = ParsedObject.filterByType(typeDeclarations, MixinDeclaration.class, mixinDecls) ;
		for(Enumeration e=d.elements(); e.hasMoreElements();)
			((TypeDeclaration)e.nextElement()).extractModuleInformation(this) ;
	}

	void typeCheck() throws JamException {
		for(Enumeration e=typeDeclarations.elements(); e.hasMoreElements();)
			((TypeDeclaration)e.nextElement()).typeCheck(this) ;
	}

}

