//
// Project : Jam
// File    : ConstructorDeclaration.java
// Author  : Giovanni Lagorio (gio@libertyline.com)
//

package jam ;

import java.util.* ;
import invisible.jacc.parse.Token ;

class ConstructorDeclaration extends MemberDeclaration {

	private Modifiers modifiers ;
	private String identifier ;
	private Vector formalParameters ;
	private Vector exceptions ;
	private Vector statements ;

	ConstructorDeclaration(Modifiers modifiers_, String identifier_, Vector formalParameters_,
							Vector exceptions_, Vector statements_,
							Token sourcePosition_) {
		super(sourcePosition_) ;
		modifiers = modifiers_ ;
		identifier = identifier_ ;
		formalParameters = formalParameters_ ;
		exceptions = exceptions_ ;
		statements = statements_ ;
	}

	Modifiers getModifiers() {
		return modifiers ;
	}

	String getIdentifier() {
		return identifier ;
	}

	Vector getFormalParameters() {
		return formalParameters ;
	}

	Vector getExceptions() {
		return exceptions ;
	}

	void dump(TabPrintStream s) throws JamException {
		s.print(modifiers.toString()) ;
		s.print(identifier+"(") ;
		dumpFP(s, formalParameters) ;
		s.print(") ") ;
		if ( exceptions.size()!=0 ) {
			s.print("throws ") ;
			dumpNames(s, exceptions) ;
		}
		s.incLevel() ;
		s.println(" {") ;
		dumpStatements(statements, s) ;
		s.decLevel() ;
		s.println("}\n") ;
	}

	void implResolveNames(CompilationUnit cu) throws JamException {
		resolveFP(formalParameters, cu) ;
		exceptions = resolveNames(exceptions, cu) ;
		resolveStatements(statements, cu) ;
	}

	void implTypeCheck(Env env, ModuleType currentModule, CompilationUnit cu) throws JamException {
		env = env.copy() ;
		for(Enumeration e=formalParameters.elements(); e.hasMoreElements(); ) {
			FormalParameter fp = (FormalParameter)e.nextElement() ;
			env.addAssociation(fp.getIdentifier(), fp.getType()) ;
		}
		typeCheckStatements(statements, env, currentModule, cu) ;
	}

	void mixinDump(TabPrintStream s) {
		throw new RuntimeException("Internal compiler error; Constructor.mixinDump") ;
	}

}
