//
// Project : Jam
// File    : FieldDeclaration.java
// Author  : Giovanni Lagorio (gio@libertyline.com)
//

package jam ;

import java.util.* ;
import invisible.jacc.parse.Token ;

class FieldDeclaration extends MemberDeclaration {

	private Modifiers modifiers ;
	private Type type ;
	private Vector declarators ;

	FieldDeclaration(Modifiers modifiers_, Type type_, Vector declarators_, Token sourcePosition_) {
		super(sourcePosition_) ;
		modifiers = modifiers_ ;
		type = type_ ;
		declarators = declarators_ ;
	}

	Modifiers getModifiers() {
		return modifiers ;
	}

	Type getType() {
		return type ;
	}

	Vector getDeclarators() {
		return declarators ;
	}

	void dump(TabPrintStream s) throws JamException {
		s.print(modifiers.toString()) ;
		s.print(type.toString()+" ") ;
		Decl.dumpVD(s, declarators) ;
		s.println(" ;"+fileInfo(sourcePosition)) ;
		s.println("") ;
	}

	void mixinDump(TabPrintStream s) {}

	void mixinDump(TabPrintStream s, Field [] fields, String fullMixinName, ClassType ct, Token sourcePosition) throws JamException {
		boolean inherited = modifiers.isSet(Modifiers.INHERITED) ;
		outer: for(Enumeration e=declarators.elements() ; e.hasMoreElements() ;) {
			VariableDeclarator vd = (VariableDeclarator)e.nextElement() ;
			VariableDeclaratorId vid = vd.getDeclaratorId() ;
			String id = vid.getIdentifier() ;
			if (inherited)
				for(int a=0; a<fields.length; ++a) {
					Field f = fields[a] ;
					if (f.getIdentifier().equals(id) && !f.getModifiers().isSet(Modifiers.INHERITED))
						continue outer ;
				}
			s.println("// Field \""+id+"\"") ;
			Type fieldType = type.addDimensions(vid.getDimensions()) ;
			if (!modifiers.isSet(Modifiers.INHERITED)) {
				s.print(modifiers.toString()) ;
				s.print(type.toString()+" ") ;
				vd.dump(s) ;
				s.println(" ;") ;
			} else if ( !ct.providesField(id, fieldType, modifiers) )
					throw new JamException("Class "+ct+" doesn't provide a field named "+
						id+" of type "+fieldType, sourcePosition) ;
			if (!modifiers.isSet(Modifiers.STATIC) && modifiers.isSet(Modifiers.PUBLIC)) {
				s.println("public "+fieldType+" "+MixinDeclaration.getterFor(id, fullMixinName)+"() {") ;
				s.println("\treturn "+id+" ;") ;
				s.println("}") ;
				dumpSettersFor(fieldType, id, s, fullMixinName) ;
			}
			s.println("") ;
		}
	}

	private static String [] setters = MixinDeclaration.setters ;
	private static String [] incrDecrSetters = MixinDeclaration.incrDecrSetters ;

	private void dumpSettersFor(Type t, String id, TabPrintStream s, String fullMixinName) {
		String fn = MixinDeclaration.mangled(fullMixinName) ;
		s.println("public "+t+" "+fn+setters[0]+id+"("+t+" newValue) {") ;
		s.println("\treturn "+id+" = newValue ;") ;
		s.println("}") ;
		if ( t==PrimitiveType.booleanInstance ) 
			dumpBooleanSettersFor(id, s, fn) ;
		else if ( t instanceof PrimitiveType )
			dumpPrimitiveTypeSettersFor(t, id, s, fn) ;
		else if ( t.toString().equals("java.lang.String") )
			dumpStringTypeSettersFor(t, id, s, fn) ;
	}

	private void dumpStringTypeSettersFor(Type t, String id, TabPrintStream s, String mangledName) {
		String [] types = PrimitiveType.names ;
		s.println("public java.lang.String "+mangledName+setters[4]+id+"(java.lang.Object rightOp) {") ;
		s.println("\treturn "+id+" += rightOp ;") ;
		s.println("}") ;
		for(int a=0; a<types.length; ++a) {
			s.println("public java.lang.String "+mangledName+setters[4]+id+"("+types[a]+" rightOp) {") ;
			s.println("\treturn "+id+" += rightOp ;") ;
			s.println("}") ;
		}
	}

	private void dumpPrimitiveTypeSettersFor(Type t, String id, TabPrintStream s, String mangledName) {
		for(int a=1; a<=11; ++a) {
			if (a>=6) // shifts && bitwise operators
				s.println("public "+t+" "+mangledName+setters[a]+id+"(int rightOp) {") ;
			else	s.println("public "+t+" "+mangledName+setters[a]+id+"(double rightOp) {") ;
			s.println("\treturn "+id+" "+BinaryOperator.operators[a+BinaryOperator.ASSIGN]
											+" rightOp ;") ;
			s.println("}") ;
		}
		for(int a=0; a<incrDecrSetters.length; ++a) {
			s.println("public "+t+" "+mangledName+incrDecrSetters[a]+id+"() {") ;
			String op = a<=1 ? "++" : "--" ;
			String appliedOp = (a&1)==0 ? op+id : id+op ;
			s.println("\treturn "+appliedOp+" ;") ;
			s.println("}") ;
		}
	}

	private void dumpBooleanSettersFor(String id, TabPrintStream s, String mangledName) {
		for(int a=9; a<=11; ++a) {
			s.println("public boolean "+mangledName+setters[a]+id+"(boolean rightOp) {") ;
			s.println("\treturn "+id+" "+BinaryOperator.operators[a+BinaryOperator.ASSIGN]
											+" rightOp ;") ;
			s.println("}") ;
		}
	}

	void implResolveNames(CompilationUnit cu) throws JamException {
		type.resolveNames(cu) ;
		resolveVD(declarators, cu) ;
	}

	void implTypeCheck(Env env, ModuleType currentModule, CompilationUnit cu) throws JamException {
		if (!modifiers.isSet(Modifiers.STATIC)) {
			env.copy() ;
			env.addAssociation("this", currentModule) ;
		}
		for(Enumeration e=declarators.elements(); e.hasMoreElements(); ) 
			((VariableDeclarator)e.nextElement()).typeCheck(env, currentModule, cu) ;
	}

}

