//
// Project : Jam
// File    : ForStatement.java
// Author  : Giovanni Lagorio (gio@libertyline.com)
//

package jam ;

import java.util.* ;
import invisible.jacc.parse.Token ;

class ForStatement extends Statement {

	private Object init ;	// can be a Vector (of comma separated StatementExpressions),
				// a LocalVariableDeclaration or just null
	private Expr guard ;	// can be null
	private Vector update ;	// can be null
	private Statement statement ;

	ForStatement(Object init_, Expr guard_, Vector update_, Statement statement_, Token sourcePosition_) {
		super(sourcePosition_) ;
		init = init_ ;
		guard = guard_ ;
		update = update_ ;
		statement = statement_ ;
	}

	public void dump(TabPrintStream s) throws JamException {
		s.print("for(") ;
		if (init!=null) {
			if (init instanceof Vector)
				s.print( Expr.toString((Vector)init) ) ;
			else	((LocalVarDeclaration)init).dump(s) ;
		}
		s.print("; ") ;
		if (guard!=null) s.print(guard+" ") ;
		s.print("; ") ;
		if (update!=null) s.print(Expr.toString(update)) ;
		s.println(")") ;
		s.incLevel() ;
		statement.dump(s) ;
		s.decLevel() ;
	}

	void implResolveNames(CompilationUnit cu) throws JamException {
		if (init!=null) {
			if (init instanceof Vector)
				ParsedObject.resolveExprs((Vector)init, cu) ;
			else	((LocalVarDeclaration)init).resolveNames(cu) ;
		}
		if (guard!=null) guard.resolveNames(cu) ;
		if (update!=null) resolveExprs(update, cu) ;
		statement.resolveNames(cu) ;
	}

	void implTypeCheck(Env env, ModuleType currentModule, CompilationUnit cu) throws JamException {
		if (init!=null) {
			if (init instanceof Vector)
				init = Expr.typeCheckVector((Vector)init, env, currentModule, cu) ;
			else	{
				env = env.copy() ;
				LocalVarDeclaration lvd = (LocalVarDeclaration)init ;
				Type base = lvd.getType() ;
				for(Enumeration e = lvd.getDeclarators() ; e.hasMoreElements() ; ) {
					VariableDeclarator vd = (VariableDeclarator)e.nextElement() ;
					vd.typeCheck(env, currentModule, cu) ;
					VariableDeclaratorId vdId = vd.getDeclaratorId() ;
					env.addAssociation(vdId.getIdentifier(), 
								base.addDimensions(vdId.getDimensions()));
				}
			}
		}
		if (guard!=null) guard = guard.typeCheck(env, currentModule, cu) ;
		if (update!=null) update = Expr.typeCheckVector(update, env, currentModule, cu) ;
		statement.typeCheck(env, currentModule, cu) ;
	}

}
