//
// Project : Jam
// File    : IfStatement.java
// Author  : Giovanni Lagorio (gio@libertyline.com)
//

package jam ;

import invisible.jacc.parse.Token ;

class IfStatement extends Statement {

	private Expr expr ;
	private Statement thenStatement, elseStatement ; // elseStatement may be null

	IfStatement(Expr expr_, Statement thenStatement_, Statement elseStatement_, Token sourcePosition_) {
		super(sourcePosition_) ;
		expr = expr_ ;
		thenStatement = thenStatement_ ;
		elseStatement = elseStatement_ ;
	}

	void dump(TabPrintStream s) throws JamException {
		s.println("if ("+expr+")") ;
		s.incLevel() ;
		thenStatement.dump(s) ;
		s.decLevel() ;
		if (elseStatement!=null) {
			s.println("else") ;
			s.incLevel() ;
			elseStatement.dump(s) ;
			s.decLevel() ;
		}
	}

	void implResolveNames(CompilationUnit cu) throws JamException {
		expr.resolveNames(cu) ;
		thenStatement.resolveNames(cu) ;
		if (elseStatement!=null)
			elseStatement.resolveNames(cu) ;
	}

	void implTypeCheck(Env env, ModuleType currentModule, CompilationUnit cu) throws JamException {
		expr = expr.typeCheck(env, currentModule, cu) ;
		thenStatement.typeCheck(env, currentModule, cu) ;
		if (elseStatement!=null)
			elseStatement.typeCheck(env, currentModule, cu) ;
	}

}
