//
// Project : Jam
// File    : InterfaceType.java
// Author  : Giovanni Lagorio (gio@libertyline.com)
//

package jam ;

import java.util.* ;

class InterfaceType extends ModuleType {

	private final static Constructor [] constructors = new Constructor[0] ;
	private InterfaceType [] superInterfaces ;

	InterfaceType(Name name_) throws JamException {
		super(name_) ;
	}

	void init(Vector superIfaces, Field [] fields_, Method [] methods_) throws JamException {
		((ModuleType)this).constructors = constructors ;
		fields = fields_ ;
		methods = methods_ ;
		superInterfaces = new InterfaceType[superIfaces.size()] ;
		for(int a=0; a<superInterfaces.length; ++a) {
			Type t = Type.getType(((Name)superIfaces.elementAt(a)).toString()) ;
			if (!(t instanceof InterfaceType))
				throw new JamException("bad 'extends' declaration; \""+t+"\" isn't an interface.");
			superInterfaces[a] = (InterfaceType)t ;
		}
	}

	InterfaceType(Class c) throws JamException {
		super(c) ;
		java.lang.Class [] ifaces = c.getInterfaces() ;
		superInterfaces = new InterfaceType[ifaces.length] ;
		for(int a=0; a<ifaces.length; ++a)
			superInterfaces[a] = (InterfaceType)Type.getType(ifaces[a]) ;
	}

	protected boolean doubleDispatch(Type that) {
		return that.isSubtypeOf(this) ;
	}

	Field getField(String identifier) throws JamException {
		for(int a=0; a<fields.length; ++a)
			if (fields[a].getIdentifier().equals(identifier))
				return fields[a] ;
		return getSuperField(identifier) ;
	}
	
	Field getSuperField(String identifier) throws JamException {
		Field found = null ;
		for(int a=0; a<superInterfaces.length; ++a) {
			Field f = superInterfaces[a].getField(identifier) ;
			if (f!=null) {
				if (found!=null)
					throw new JamException("identifier \""+identifier+"\" is ambiguous");
				found = f ;
			}
		}
		return found ;
	}

	boolean isSubtypeOf(ClassType ct) {
		return ct.toString().equals("java.lang.Object") ;
	}

	boolean isSubtypeOf(InterfaceType it) {
		if (toString()==it.toString()) return true ;
		for(int a=0; a<superInterfaces.length; ++a) 
			if (superInterfaces[a].isSubtypeOf(it)) return true ;
		return false ;
	}

	Vector getSuperApplicableMethods(String methodName, Type [] args) throws JamException {
		Vector v = new Vector() ;
		for(int a=0; a<superInterfaces.length; ++a) {
			Vector vsuper = superInterfaces[a].getApplicableMethods(methodName, args) ;
			for(Enumeration e=vsuper.elements(); e.hasMoreElements() ;)
				v.addElement(e.nextElement()) ;
		}
		return v ;
	}

	Vector getApplicableMethods(String methodName, Type [] args) throws JamException {
		Vector appMethods = getSuperApplicableMethods(methodName, args) ;
		for(int a=0; a<methods.length; ++a) {
			Method m = methods[a] ;
			if ( m.getIdentifier().equals(methodName) && subtype(args, m.getArguments()) )
				appMethods.addElement(m) ;
		}
		return appMethods ;
	}

	private void checkCycles(InterfaceType it, Hashtable visited) throws JamException {
		if (it==this) 
			throw new JamException("interface \""+moduleName+"\": cyclic 'extends' relation.");
		if (visited.get(it)!=null) return ;
		visited.put(it, it) ;
		for(int a=0; a<it.superInterfaces.length; ++a)
			checkCycles(it.superInterfaces[a], visited) ;
	}

	void checkCycles() throws JamException {
		for(int a=0; a<superInterfaces.length; ++a)
			checkCycles(superInterfaces[a], new Hashtable()) ;
	}

}

