//
// Project : Jam
// File    : JamClassDeclaration.java
// Author  : Giovanni Lagorio (gio@libertyline.com)
//

package jam ;

import java.util.* ;
import invisible.jacc.parse.Token ;

class JamClassDeclaration extends TypeDeclaration {

	private Modifiers modifiers ;
	private Name mixin, superclass ;

	JamClassDeclaration(Modifiers modifiers_, String identifier_, Name superclass_, Name mixin_,
								Vector memberDecls_, Token sourcePosition_) {
		super(sourcePosition_, identifier_, memberDecls_) ;
		modifiers = modifiers_ ;
		mixin = mixin_ ;
		superclass = superclass_ ;
		memberDecls = memberDecls_ ;
	}

	void dump(TabPrintStream s) throws JamException {
		s.print(modifiers.toString()) ;
		s.println("class "+identifier+" extends "+superclass+" implements "+mixin+" {") ;
		s.incLevel() ;
		Field [] fields = mt.fields ;
		String fullName = mixin.toString() ;
		for(Enumeration e=mt.getMemberDeclarations(); e.hasMoreElements() ;) {
			MemberDeclaration md = (MemberDeclaration)e.nextElement() ;
			if (md instanceof FieldDeclaration) {
				FieldDeclaration fd = (FieldDeclaration)md ;
				fd.mixinDump(s, fields, fullName, sct, sourcePosition) ;
			} else	{
				MethodDeclaration methd = (MethodDeclaration)md ;
				MethodHeader header = methd.getHeader() ;
				Modifiers mod = header.getModifiers() ;
				String mname = header.getIdentifier() ;
				if ( mod.isSet(Modifiers.INHERITED) && !sct.providesMethod(mname,
								header.getParametersType(),
								header.getReturnType(),
								null, // Exceptions clause .... to do!!! 
								mod) )
					throw new JamException("Class "+sct+" doesn't provide "+
							"a method "+mname+" of correct type.", sourcePosition) ;
				md.mixinDump(s) ;
			}
		}
		dumpDecls(s, memberDecls) ;
		s.decLevel() ;
		s.println("}\n") ;
	}

	void implResolveNames(CompilationUnit cu) throws JamException {
		mixin = cu.getFullyQualifiedName(mixin) ;
		superclass = cu.getFullyQualifiedName(superclass) ;
		resolveDecls(memberDecls, cu) ;
	}

	private ClassType ct, sct ;
	private MixinType mt ;

	void createModuleObject(CompilationUnit cu) throws JamException {
		super.createModuleObject(cu) ;
		ct = new ClassType(fullName) ;
	}

	void extractModuleInformation(CompilationUnit cu) throws JamException {
		final Type t = Type.getType(mixin.toString()) ;
		if (!(t instanceof MixinType))
			throw new JamException(t+" is not a mixin.") ;
		mt = (MixinType)t ;
		Field [] mf = mt.fields ;
		int definedFields=0 ;
		for(int a=0; a<mf.length; ++a)
			if ( !mf[a].getModifiers().isSet(Modifiers.INHERITED) )
				++definedFields ;
		Field [] fields = new Field[definedFields] ;
		for(int a=0, b=0; a<mf.length; ++a)
			if ( !mf[a].getModifiers().isSet(Modifiers.INHERITED) )
				fields[b++] = mf[a] ;
		Method [] mm = mt.methods ;
		int definedMethods=0 ;
		for(int a=0; a<mm.length; ++a)
			if ( !mm[a].getModifiers().isSet(Modifiers.INHERITED) )
				++definedMethods ;
		Method [] methods = new Method[definedMethods] ;
		for(int a=0, b=0; a<mm.length; ++a)
			if ( !mm[a].getModifiers().isSet(Modifiers.INHERITED) )
				methods[b++] = mm[a] ;
		ct.init(superclass.toString(), new Vector(), constructors(ct), fields, methods, mt) ;
		sct = (ClassType)Type.getType(superclass.toString()) ;
	}

}
