//
// Project : Jam
// File    : MethodDeclaration.java
// Author  : Giovanni Lagorio (gio@libertyline.com)
//

package jam ;

import java.util.* ;

class MethodDeclaration extends MemberDeclaration {

	private MethodHeader header ;
	private Block body ; // may be null (e.g. in case of abstract methods)

	MethodDeclaration(MethodHeader header_, Block body_) {
		super(header_.getSourcePosition()) ;
		header = header_ ;
		body = body_ ;
	}

	MethodHeader getHeader() {
		return header ;
	}

	void dump(TabPrintStream s) throws JamException {
		header.dump(s) ;
		if (body!=null)
			body.dump(s, true) ;
		else	s.println("; // line: "+header.getSourcePosition().line) ;
		s.println("") ;
	}

	void mixinDump(TabPrintStream s) throws JamException {
		Modifiers m = header.getModifiers() ;
		if ( m.isSet(Modifiers.INHERITED) )
			return ;
		header.dump(s) ;
		if (body!=null)
			body.dump(s, true) ;
		else	s.println(";") ;
		s.println("") ;
	}

	void implResolveNames(CompilationUnit cu) throws JamException {
		header.resolveNames(cu) ;
		if (body!=null)
			body.resolveNames(cu) ;
	}

	void implTypeCheck(Env env, ModuleType currentModule, CompilationUnit cu) throws JamException {
		env = env.copy() ;
		for(Enumeration e=header.getFormalParameters().elements(); e.hasMoreElements(); ) {
			FormalParameter fp = (FormalParameter)e.nextElement() ;
			env.addAssociation(fp.getIdentifier(), fp.getType()) ;
		}
		if (!header.getModifiers().isSet(Modifiers.STATIC))
			env.addAssociation("this", currentModule) ;
		if (body!=null)
			body.typeCheck(env, currentModule, cu) ;
	}

}
