//
// Project : Jam
// File    : MethodHeader.java
// Author  : Giovanni Lagorio (gio@libertyline.com)
//

package jam ;

import java.util.Vector ;
import invisible.jacc.parse.Token ;

class MethodHeader {

	private Modifiers modifiers ;
	private Type type ;
	private String identifier ;
	private Vector formalParameters ;
	private Vector exceptions ;
	private Token sourcePosition ;

	Type [] getParametersType() {
		int n = formalParameters.size() ;
		Type [] t = new Type[n] ;
		for(int a=0; a<n; ++a) {
			FormalParameter fp = (FormalParameter)formalParameters.elementAt(a) ;
			t[a] = fp.getType() ;
		}
		return t ;
	}

	MethodHeader(Modifiers modifiers_, Type type_, String identifier_, Vector formalParameters_,
							Vector exceptions_, Token sourcePosition_) {
		modifiers = modifiers_ ;
		type = type_ ;
		identifier = identifier_ ;
		formalParameters = formalParameters_ ;
		exceptions = exceptions_ ;
		sourcePosition = sourcePosition_ ;
	}

	Modifiers getModifiers() {
		return modifiers ;
	}

	String getIdentifier() {
		return identifier ;
	}

	Vector getFormalParameters() {
		return formalParameters ;
	}

	Vector getExceptions() {
		return exceptions ;
	}

	Type getReturnType() {
		return type ;
	}

	Token getSourcePosition() { return sourcePosition ; }

	void dump(TabPrintStream s) throws JamException {
		s.print(modifiers.toString()) ;
		s.print(type.toString()+" ") ;
		s.print(identifier+"(") ;
		ParsedObject.dumpFP(s, formalParameters) ;
		s.print(") ") ;
		if ( exceptions.size()!=0 ) {
			s.print("throws ") ;
			Decl.dumpNames(s, exceptions) ;
			s.print(" ") ;
		}
	}

	void resolveNames(CompilationUnit cu) throws JamException {
		type.resolveNames(cu) ;
		ParsedObject.resolveFP(formalParameters, cu) ;
		exceptions = ParsedObject.resolveNames(exceptions, cu) ;
	}

}

