//
// Project : Jam
// File    : MixinDeclaration.java
// Author  : Giovanni Lagorio (gio@libertyline.com)
//

package jam ;

import java.util.Vector ;
import invisible.jacc.parse.Token ;

class MixinDeclaration extends TypeDeclaration {

	private Vector interfaces ;
	
	MixinDeclaration(String identifier_, Vector memberDecls_, Vector interfaces_, Token sourcePosition_) {
		super(sourcePosition_, identifier_, memberDecls_) ;
		interfaces = interfaces_ ;
	}

	static String mangled(String s) {
		StringBuffer sb = new StringBuffer() ;
		int l = s.length() ;
		for(int a=0; a<l; ++a) {
			char c = s.charAt(a) ;
			sb.append( c=='.' ? '$' : c) ;
		}
		return sb.toString() ;
	}

	static String getterFor(String identifier, String typename) {
		return mangled(typename)+"_$get$_"+identifier ;
	}

	static final String [] setters = {
		/*  0 */ "_$set$_",
		/*  1 */ "_$multiplyAndSet$_",
		/*  2 */ "_$divideAndSet$_",
		/*  3 */ "_$remainderAndSet$_",
		/*  4 */ "_$addAndSet$_",
		/*  5 */ "_$subtractAndSet$_",
		/*  6 */ "_$leftShiftAndSet$_",
		/*  7 */ "_$signedRightShiftAndSet$_",
		/*  8 */ "_$unsignedRightShiftAndSet$_",
		/*  9 */ "_$andAndSet$_",
		/* 10 */ "_$exclusiveOrAndSet$_",
		/* 11 */ "_$inclusiveOrAndSet$_",
	} ;

	static final String [] incrDecrSetters = {
		/*  0 */ "_$prefix_increment$_",
		/*  1 */ "_$postfix_increment$_",
		/*  2 */ "_$prefix_decrement$_",
		/*  3 */ "_$postfix_decrement$_",
	} ;

	void dump(TabPrintStream s) throws JamException {
		dumpImpl(s) ;
	}

	private void dumpBooleanSettersFor(Field f, TabPrintStream s) throws JamException {
		String fn = mangled(fullName.toString()) ;
		for(int a=9; a<=11; ++a)
			s.println("boolean "+fn+setters[a]+f.getIdentifier()+"(boolean rightOp) ;") ;
	}

	private void dumpPrimitiveTypeSettersFor(Field f, TabPrintStream s) throws JamException {
		Type t = f.getType() ;
		String fn = mangled(fullName.toString()) ;
		for(int a=1; a<=11; ++a)
			if (a>=6) // shifts && bitwise operators
				s.println(t+" "+fn+setters[a]+f.getIdentifier()+"(int rightOp) ;") ;
			else	s.println(t+" "+fn+setters[a]+f.getIdentifier()+"(double rightOp) ;") ;
		for(int a=0; a<incrDecrSetters.length; ++a)
			s.println(t+" "+fn+incrDecrSetters[a]+f.getIdentifier()+"() ;") ;
	}

	private void dumpStringTypeSettersFor(Field f, TabPrintStream s) throws JamException {
		String [] types = PrimitiveType.names ;
		String fn = mangled(fullName.toString()) ;
		s.println("java.lang.String "+fn+setters[4]+f.getIdentifier()+"(java.lang.Object rightOp) ;") ;
		for(int a=0; a<types.length; ++a)
			s.println("java.lang.String "+fn+setters[4]+f.getIdentifier()+"("+types[a]+" rightOp) ;") ;
	}

	private void dumpSettersFor(Field f, TabPrintStream s) throws JamException {
		Type t = f.getType() ;
		String fn = mangled(fullName.toString()) ;
		s.println(t+" "+fn+setters[0]+f.getIdentifier()+"("+t+" newValue) ;") ;
		if ( t==PrimitiveType.booleanInstance ) 
			dumpBooleanSettersFor(f, s) ;
		else if ( t instanceof PrimitiveType )
			dumpPrimitiveTypeSettersFor(f, s) ;
		else if ( t.toString().equals("java.lang.String") )
			dumpStringTypeSettersFor(f, s) ;
		s.println("") ;
	}

	private void dumpFields(TabPrintStream s, boolean inherited) throws JamException {
		boolean doLf = false ;
		String fn = fullName.toString() ;
		outer: for(int a=0; a<mt.fields.length; ++a) {
			Field f = mt.fields[a] ;
			Modifiers m = f.getModifiers() ;
			if ( m.isSet(Modifiers.INHERITED)!=inherited || !m.isSet(Modifiers.PUBLIC) )
				continue ;
			String fid = f.getIdentifier() ;
			if ( inherited )
				for(int b=0; b<mt.fields.length; ++b) {
					Field otherF = mt.fields[b] ;
					if (otherF.getIdentifier().equals(fid) && !otherF.getModifiers().isSet(Modifiers.INHERITED))
						continue outer ;
				}
			doLf = true ;
			s.println("// Field \""+fid+"\"") ;
			s.println(f.getType()+" "+getterFor(fid, fn)+"() ;") ;
			dumpSettersFor(f, s) ;
		}
		if (doLf) s.println("") ;
	}

	private void dumpMethods(TabPrintStream s, boolean inherited) throws JamException {
		s.println("// Methods:") ;
		for(int a=0; a<mt.methods.length; ++a) {
			Method m = mt.methods[a] ;
			Modifiers mod = m.getModifiers() ;
			if ( mod.isSet(Modifiers.INHERITED)!=inherited || !mod.isSet(Modifiers.PUBLIC) )
				continue ;
			MethodDeclaration md = m.getDeclaration() ;
			md.getHeader().dump(s) ;
			s.println(";") ;
		}
	}

	private void dumpImpl(TabPrintStream s) throws JamException {
		s.println("interface Parent$"+identifier+" {\n") ;
		s.incLevel() ;
		dumpFields(s, true) ;
		dumpMethods(s, true) ;
		s.decLevel() ;
		s.println("\n}\n") ;
		s.print("public interface "+identifier+" extends ") ;
		dumpNames(s, interfaces) ;
		s.println(" {\n") ;
		s.incLevel() ;
		dumpFields(s, false) ;
		dumpMethods(s, false) ;
		s.decLevel() ;
		s.println("\n}\n") ;

	}

	void implResolveNames(CompilationUnit cu) throws JamException {
		resolveDecls(memberDecls, cu) ;
		interfaces = resolveNames(interfaces, cu) ;
	}

	private MixinType mt ;

	void createModuleObject(CompilationUnit cu) throws JamException {
		super.createModuleObject(cu) ;
		mt = new MixinType(fullName, memberDecls) ;
	}

	void extractModuleInformation(CompilationUnit cu) throws JamException {
		mt.init( fields(mt), methods(mt), interfaces ) ;
		interfaces.insertElementAt(new Name("Parent$"+identifier), 0) ;
	}

}
