//
// Project : Jam
// File    : MixinType.java
// Author  : Giovanni Lagorio (gio@libertyline.com)
//

package jam ;

import java.util.* ;

class MixinType extends ModuleType {

	private final static Constructor [] constructors = new Constructor[0] ;
	private Vector memberDeclarations ;
	private InterfaceType [] interfaces ;

	MixinType(Name name_, Vector memberDeclarations_) throws JamException {
		super(name_) ;
		((ModuleType)this).constructors = constructors ;
		memberDeclarations = memberDeclarations_ ;
	}

	Enumeration getMemberDeclarations() {
		return memberDeclarations.elements() ;
	}

	void init(Field [] fields_, Method [] methods_, Vector ifacesV) throws JamException {
		fields = fields_ ;
		methods = methods_ ;
		ClassType object = null ;
		try	{
			object = (ClassType)Type.getType("java.lang.Object") ;	
		} catch(JamException je) {}
		for(int a=0; a<methods.length; ++a) {
			Modifiers mod = methods[a].getModifiers() ;
			if ( mod.isSet(Modifiers.INHERITED) )
				methods[a].declaringModule = object ;
		}
		interfaces = new InterfaceType[ifacesV.size()] ;
		for(int a=0; a<interfaces.length; ++a) {
			String i = ((Name)ifacesV.elementAt(a)).toString() ;
			Type t = Type.getType(i) ;
			if (!(t instanceof InterfaceType))
				throw new JamException("bad 'implements' declaration; \""+i+"\""+
								" isn't an interface.") ;
			interfaces[a] = (InterfaceType)t ;
		}
	}

	protected boolean doubleDispatch(Type that) {
		return that.isSubtypeOf(this) ;
	}

	Field getField(String identifier) {
		Field f = implGetField(identifier) ;
		if ( f==null ) return null ;
		if ( f.getModifiers().isSet(Modifiers.STATIC) ) return null ;
		return f ;
	}

	private Field implGetField(String identifier) {
		for(int a=0; a<fields.length; ++a) {
			Field f = fields[a] ;
			Modifiers m = f.getModifiers() ;
			if (f.getIdentifier().equals(identifier) && !m.isSet(Modifiers.INHERITED))
				return f ;
		}
		for(int a=0; a<fields.length; ++a) {
			Field f = fields[a] ;
			Modifiers m = f.getModifiers() ;
			if (f.getIdentifier().equals(identifier))
				return f ;
		}
		return null ;
	}

	Field getSuperField(String identifier) {
		for(int a=0; a<fields.length; ++a) {
			Field f = fields[a] ;
			Modifiers m = f.getModifiers() ;
			if (f.getIdentifier().equals(identifier) && m.isSet(Modifiers.INHERITED) &&
									!m.isSet(Modifiers.STATIC) )
				return f ;
		}
		return null ;
	}

	Vector getSuperApplicableMethods(String methodName, Type [] args) throws JamException {
		Vector appMethods = new Vector() ;
		outer: for(int a=0; a<methods.length; ++a) {
			Method m = methods[a] ;
			Modifiers mod = m.getModifiers() ;
			if ( !mod.isSet(Modifiers.INHERITED) || mod.isSet(Modifiers.STATIC) )
				continue ;
			if ( m.getIdentifier().equals(methodName) && subtype(args, m.getArguments()) )
				appMethods.addElement(m) ;
		}
		return appMethods ;
	}

	Vector getApplicableMethods(String methodName, Type [] args) throws JamException {
		//System.out.println("MixinType.getApplicableMethods(methodName="+methodName+", args="+args+", args.l="+args.length+")") ;
		//for(int a=0; a<args.length; ++a)
		//	System.out.println("args["+a+"]="+args[a]) ;
		Vector appMethods = new Vector() ;
		for(int a=0; a<methods.length; ++a) {
			Method m = methods[a] ;
			if ( m.getModifiers().isSet(Modifiers.STATIC) ) continue ;
			if ( m.getIdentifier().equals(methodName) && subtype(args, m.getArguments()) )
				appMethods.addElement(m) ;
		}
		return appMethods ;
	}

	boolean isSubtypeOf(ClassType ct) {
		return ct.toString().equals("java.lang.Object") ;
	}

	boolean isSubtypeOf(MixinType mt) {
		return toString().equals(mt.toString()) ;
	}

	void checkCycles() throws JamException {}
}

