//
// Project : Jam
// File    : Modifiers.java
// Author  : Giovanni Lagorio (gio@libertyline.com)
//

package jam ;

class Modifiers {
	
	// standard bit flags
	static final int PUBLIC		= 0x0001;
	static final int PRIVATE	= 0x0002;
	static final int PROTECTED	= 0x0004;
	static final int STATIC		= 0x0008;
	static final int FINAL		= 0x0010;
	static final int SYNCHRONIZED	= 0x0020;
	static final int VOLATILE	= 0x0040;
	static final int TRANSIENT	= 0x0080;
	static final int NATIVE		= 0x0100;
	static final int INTERFACE	= 0x0200; // Not used
	static final int ABSTRACT	= 0x0400;
	// jam bit flags
	static final int INHERITED	= 0x0800; 
	
	private int flags ;

	Modifiers() {}
	Modifiers(int flags_) { flags = flags_ ; }

	Modifiers addFlags(int otherFlags) {
		flags |= otherFlags ;
		return this ;
	}

	int flags() {
		return flags ;
	}

	boolean isSet(int aFlag) {
		return (flags & aFlag)==aFlag ;
	}
	
	public String toString() {
		StringBuffer buf = new StringBuffer() ;
		if ((flags & PUBLIC) != 0)
			buf.append ("public ");
		if ((flags & PROTECTED) != 0)
			buf.append ("protected ");
		if ((flags & PRIVATE) != 0)
			buf.append ("private ");
		if ((flags & STATIC) != 0)
			buf.append ("static ");
		if ((flags & ABSTRACT) != 0)
			buf.append ("abstract ");
		if ((flags & FINAL) != 0)
			buf.append ("final ");
		if ((flags & NATIVE) != 0)
			buf.append ("native ");
		if ((flags & SYNCHRONIZED) != 0)
			buf.append ("synchronized ");
		if ((flags & TRANSIENT) != 0)
			buf.append ("transient ");
		if ((flags & VOLATILE) != 0)
			buf.append ("volatile ");
		// ("inherited" is _never_ used in a java file)
		return buf.toString() ;
	}
	
}


