//
// Project : Jam
// File    : NameExpr.java
// Author  : Giovanni Lagorio (gio@libertyline.com)
//

package jam ;

class NameExpr extends Expr {

	private Name name ;

	NameExpr(Name name_) {
		name = name_ ;
	}

	Name getName() {
		return name ;
	}

	public String toString() {
		return name.toString() ;
	}

	void resolveNames(CompilationUnit cu) throws JamException {}

	private static final boolean DEBUG = false ;

	static Expr typeOf(Name name, Env env, ModuleType currentModule, CompilationUnit cu)
										throws JamException {
		if (DEBUG)
			System.out.println("typeOf() name="+name) ;
		if ( name.isSimpleName() ) {
			Name fullQ = null ;
			try	{
				fullQ = cu.getFullyQualifiedName(name) ;
			} catch (JamException je) {}
			if (fullQ!=null)
				try	{
					ModuleType mt = (ModuleType)Type.getType(fullQ.toString()) ;
					Expr expr = new NameExpr(fullQ) ;
					expr.type = mt ;
					return expr ;
				} catch (ClassNotFound cnf) {}
			String identifier = name.toString() ;
			if (identifier.equals("this") && currentModule instanceof MixinType)
				throw new JamException("\"this\" keyword cannot be used inside mixins.") ;
			Type type = env.getTypeOf(identifier) ;
			if (type!=null) {
				Expr expr = new NameExpr(name) ;
				expr.type = type ;
				if (DEBUG)
					System.out.println("(1) expr.type="+expr.type) ;
				return expr ;
			}
			Field f = currentModule.getField(identifier) ;
			if (f!=null) {
				Expr expr = new NameExpr(name) ;
				expr.type = f.getType() ;
				if (DEBUG)
					System.out.println("(2) f.getType()="+expr.type) ;
				if ( env.getTypeOf("this")==null &&
							!f.getModifiers().isSet(Modifiers.STATIC) )
					throw new JamException("you cannot access the instance field \""+
										identifier+"\"") ;
				return expr ;
			}
			throw new JamException("\""+identifier+"\" not found in this environment.") ;
		} else	{
			Name firstPart = name.firstPart() ;
			String identifier = name.lastComponent() ;
			Name fullQ = null ;
			try	{
				fullQ = cu.getFullyQualifiedName(firstPart) ;
			} catch (JamException je) {}
			if (fullQ!=null)
				try	{
					ModuleType mt = (ModuleType)Type.getType(fullQ.toString()) ;
					if (DEBUG)
						System.out.println("(3) Static field access fullQ="+fullQ) ;
					return new StaticFieldAccess(mt, identifier).typeCheck(env,
										currentModule, cu) ;
				} catch (ClassNotFound cnf) {}
			Expr e = typeOf(firstPart, env, currentModule, cu) ;
			if (DEBUG)
				System.out.println("typeOf(\""+firstPart+"\")="+e+", e.type="+e.getType()) ;
			Type type = e.getType() ;
			if ( (type instanceof ModuleType) || (type instanceof ArrayType &&
									identifier.equals("length")) ) { 
				e = new PrimaryFieldAccess(e, identifier) ;
				return e.typeCheck(env, currentModule, cu) ;
			}
			throw new JamException("\""+type+"\" hasn't a field named \""+identifier+"\".") ;
		}
	}

	Expr typeCheck(Env env, ModuleType currentModule, CompilationUnit cu) throws JamException {
		if (type!=null) return this ;
		if (DEBUG)
			System.out.println("\n\nNameExpr.typeCheck name="+name) ;
		Expr e = typeOf(name, env, currentModule, cu) ;
		if (DEBUG)
			System.out.println("typeOf(\""+name+"\")="+e+", e.type="+e.getType()) ;
		if (e==null)
			throw new JamException("\""+name+"\" is either a misplaced package name or a " +
				"non-existent entity. An expression name is expected in this context.") ;
		return e ;
	}

}

