//
// Project : Jam
// File    : ParsedObject.java
// Author  : Giovanni Lagorio (gio@libertyline.com)
//

package jam ;

import java.util.* ;
import invisible.jacc.parse.Token ;

abstract class ParsedObject {

	protected Token sourcePosition ;

	ParsedObject(Token sourcePosition_) {
		sourcePosition = sourcePosition_ ;
	}

	abstract void dump(TabPrintStream s) throws JamException ;

	abstract void implResolveNames(CompilationUnit cu) throws JamException ;

	final void resolveNames(CompilationUnit cu) throws JamException {
		try	{
			implResolveNames(cu) ;
		} catch (JamException e) {
			e.setToken(sourcePosition) ;
			throw e ;
		}
	}

	// some utility functions:
	static void resolveDecls(Vector v, CompilationUnit cu) throws JamException {
		for(Enumeration e = v.elements() ; e.hasMoreElements() ;) 
			((Decl)e.nextElement()).resolveNames(cu) ;
	}

	static void resolveFP(Vector v, CompilationUnit cu) throws JamException {
		for(Enumeration e = v.elements() ; e.hasMoreElements() ;) 
			((FormalParameter)e.nextElement()).resolveNames(cu) ;
	}

	static Vector resolveNames(Vector v, CompilationUnit cu) throws JamException {
		if (v.size()==0) return v ; // avoid useless allocations
		Vector rv = new Vector() ;
		for(Enumeration e = v.elements() ; e.hasMoreElements() ;) 
			rv.addElement(cu.getFullyQualifiedName((Name)e.nextElement())) ;
		return rv ;
	}

	static void resolveStatements(Vector v, CompilationUnit cu) throws JamException {
		for(Enumeration e = v.elements() ; e.hasMoreElements() ;) 
			((Statement)e.nextElement()).resolveNames(cu) ;
	}

	static void resolveExprs(Vector v, CompilationUnit cu) throws JamException {
		for(Enumeration e = v.elements() ; e.hasMoreElements() ;) 
			((Expr)e.nextElement()).resolveNames(cu) ;
	}

	static void resolveVD(Vector v, CompilationUnit cu) throws JamException {
		for(Enumeration e = v.elements() ; e.hasMoreElements() ;) 
			((VariableDeclarator)e.nextElement()).resolveNames(cu) ;
	}

	static void dumpVD(TabPrintStream s, Vector v) throws JamException {
		boolean firstTime = true ;
		for(Enumeration e = v.elements() ; e.hasMoreElements() ; firstTime=false) {
			if (!firstTime) s.print(", ") ;
			((VariableDeclarator)e.nextElement()).dump(s) ;
		}
	}

	static void dumpFP(TabPrintStream s, Vector v) throws JamException {
		boolean firstTime = true ;
		for(Enumeration e = v.elements() ; e.hasMoreElements() ; firstTime=false) {
			if (!firstTime) s.print(", ") ;
			((FormalParameter)e.nextElement()).dump(s) ;
		}
	}

	static void dumpNames(TabPrintStream s, Vector v) {
		boolean firstTime = true ;
		for(Enumeration e = v.elements() ; e.hasMoreElements() ; firstTime=false) {
			if (!firstTime) s.print(", ") ;
			s.print(((Name)e.nextElement()).toString()) ;
		}
	}

	static void dumpDecls(TabPrintStream s, Vector v) throws JamException {
		for(Enumeration e = v.elements() ; e.hasMoreElements() ;) 
			((Decl)e.nextElement()).dump(s) ;
	}

	static void dumpStatements(Vector v, TabPrintStream s) throws JamException {
		for(Enumeration e = v.elements() ; e.hasMoreElements() ;) 
			((Statement)e.nextElement()).dump(s) ;
	}

	/*
	static int countByType(Vector v, Class c) {
		int n = 0 ;
		for(Enumeration e = v.elements() ; e.hasMoreElements() ;) 
			if ( c.isInstance(e.nextElement()) )
				++n ;
		return n ;
	}
	*/

	static Vector filterByType(Vector v, Class c, boolean positiveTest) {
		if (v.size()==0) return v ; // avoid useless allocations
		Vector rv = new Vector() ;
		for(Enumeration e = v.elements() ; e.hasMoreElements() ;) {
			Object o = e.nextElement() ;
			if ( c.isInstance(o)==positiveTest )
				rv.addElement(o) ;
		}
		return rv ;
	}

	static void copyArray(Vector v, Object [] array) {
		if (v.size()!=array.length)
			throw new RuntimeException("Internal compiler error (copyArray v.s="+v.size()+
										", a.l="+array.length+")") ;
		int i = 0 ;
		for(Enumeration e = v.elements() ; e.hasMoreElements() ;) 
			array[i++] = e.nextElement() ;
	}

	static void typeCheckStatements(Vector v, Env env, ModuleType currentModule, CompilationUnit cu) throws JamException {
		for(Enumeration e = v.elements() ; e.hasMoreElements() ;) 
			((Statement)e.nextElement()).typeCheck(env, currentModule, cu) ;
	}

	protected static String fileInfo(Token t) {
		return " // file: "+t.file+", line: "+t.line ;
	}

}

