//
// Project : Jam
// File    : PrimaryFieldAccess.java
// Author  : Giovanni Lagorio (gio@libertyline.com)
//

package jam ;

class PrimaryFieldAccess extends FieldAccess {

	private Expr primary ;

	PrimaryFieldAccess(Expr primary_, String identifier_) {
		super(identifier_) ;
		primary = primary_ ;
	}

	public String toString() {
		return primary+"."+identifier ;
	}

	void resolveNames(CompilationUnit cu) throws JamException {
		primary.resolveNames(cu) ;
	}

	Expr typeCheck(Env env, ModuleType currentModule, CompilationUnit cu) throws JamException {
		primary = primary.typeCheck(env, currentModule, cu) ;
		Type t = primary.getType() ;
		if (t instanceof ModuleType) {
			Field f = ((ModuleType)t).getField(identifier) ;
			if (f!=null) {
				if (t instanceof MixinType) {
					Expr e = new MixinFieldAccess(primary, identifier) ;
					e.type = f.getType() ;
					return e ;
				}
				type = f.getType() ;
				return this ;
			}
		}
		if (t instanceof ArrayType && identifier.equals("length")) {
			type = PrimitiveType.intInstance ;
			return this ;
		}
		throw new JamException("\""+t+"\" hasn't a field named \""+identifier+"\".") ;
	}

}

