//
// Project : Jam
// File    : PrimitiveType.java
// Author  : Giovanni Lagorio (gio@libertyline.com)
//

package jam ;

class PrimitiveType extends Type {

	private final static int BOOLEAN	= 0 ;
	private final static int BYTE		= 1 ;
	private final static int SHORT		= 2 ;
	private final static int CHAR		= 3 ;
	private final static int INT		= 4 ;
	private final static int LONG		= 5 ;
	private final static int FLOAT		= 6 ;
	private final static int DOUBLE		= 7 ;

	final static String [] names = {
		"boolean", "byte", "short", "char", "int", "long", "float", "double"
	} ;

	private int kind ; // you can leave out "final" if javac complains about this variable

	PrimitiveType(int kind_) {
		kind = kind_ ;
		try	{
			Type.newType(names[kind], this) ;
		} catch (JamException je) {
			throw new RuntimeException("Internal compiler error in PrimitiveType()") ;
		}
	} 

	protected boolean doubleDispatch(Type that) {
		return that.isSubtypeOf(this) ;
	}

	final static PrimitiveType booleanInstance = new PrimitiveType(BOOLEAN) ;
	final static PrimitiveType byteInstance = new PrimitiveType(BYTE) ;
	final static PrimitiveType shortInstance = new PrimitiveType(SHORT) ;
	final static PrimitiveType charInstance = new PrimitiveType(CHAR) ;
	final static PrimitiveType intInstance = new PrimitiveType(INT) ;
	final static PrimitiveType longInstance = new PrimitiveType(LONG) ;
	final static PrimitiveType floatInstance = new PrimitiveType(FLOAT) ;
	final static PrimitiveType doubleInstance = new PrimitiveType(DOUBLE) ;

	boolean isSubtypeOf(PrimitiveType that) {
		if (this==booleanInstance)
			return this==that ;
		if (this==byteInstance || this==shortInstance)
			return that!=charInstance && kind<=that.kind ;
		return kind<=that.kind ;
	}

	public String toString() {
		return names[kind] ;
	}

	void resolveNames(CompilationUnit cu) throws JamException {}

}

