//
// Project : Jam
// File    : SwitchStatement.java
// Author  : Giovanni Lagorio (gio@libertyline.com)
//

package jam ;

import java.util.* ;
import invisible.jacc.parse.Token ;

class SwitchStatement extends Statement {

	private Expr expr ;
	private Vector labelsAndStatements ;
	private Vector labels ;

	SwitchStatement(Expr expr_, Vector labelsAndStatements_, Vector labels_, Token sourcePosition_) {
		super(sourcePosition_) ;
		expr = expr_ ;
		labelsAndStatements = labelsAndStatements_ ;
		labels = labels_ ;
	}

	private void dumpLabels(Vector v, TabPrintStream s) throws JamException {
		for(Enumeration e=v.elements(); e.hasMoreElements();) 
			((SwitchLabel)e.nextElement()).dump(s) ;
	}

	void dump(TabPrintStream s) throws JamException {
		s.println("switch ("+expr+") {") ;
		for(Enumeration e=labelsAndStatements.elements(); e.hasMoreElements();) {
			Vector [] v = (Vector [])e.nextElement() ;
			Vector labels = v[0] ;
			Vector statements = v[1] ;
			dumpLabels(labels, s) ;
			s.incLevel() ;
			dumpStatements(statements, s) ;
			s.decLevel() ;
		}
		dumpLabels(labels, s) ;
		s.println("}") ;
	}

	void implResolveNames(CompilationUnit cu) throws JamException {
		expr.resolveNames(cu) ;
		for(Enumeration e=labelsAndStatements.elements(); e.hasMoreElements();) {
			Vector [] v = (Vector [])e.nextElement() ;
			resolveStatements(v[1], cu) ;
		}
	}

	void implTypeCheck(Env env, ModuleType currentModule, CompilationUnit cu) throws JamException {
		expr = expr.typeCheck(env, currentModule, cu) ;
		for(Enumeration e=labelsAndStatements.elements(); e.hasMoreElements();) {
			Vector [] v = (Vector [])e.nextElement() ;
			typeCheckStatements(v[1], env, currentModule, cu) ;
		}
	}

}

