//
// Project : Jam
// File    : TryStatement.java
// Author  : Giovanni Lagorio (gio@libertyline.com)
//

package jam ;

import java.util.* ;

class TryStatement extends Statement {

	private Block block ;
	private Vector catches ;
	private Block finallyBlock ; // may be null (if there's no finally block)

	TryStatement(Block block_, Vector catches_, Block finallyBlock_) {
		super(null) ;
		block = block_ ;
		catches = catches_ ;
		finallyBlock = finallyBlock_ ;
	}

	public void dump(TabPrintStream s) throws JamException {
		s.print("try ") ;
		block.dump(s, true) ;
		for(Enumeration e = catches.elements() ; e.hasMoreElements() ;) 
			((CatchClause)e.nextElement()).dump(s) ;
		if (finallyBlock!=null) {
			s.print("finally ") ;
			finallyBlock.dump(s, true) ;
		}
	}

	void implResolveNames(CompilationUnit cu) throws JamException {
		block.resolveNames(cu) ;
		if (finallyBlock!=null)
			finallyBlock.resolveNames(cu) ;
		for(Enumeration e = catches.elements() ; e.hasMoreElements() ;) 
			((CatchClause)e.nextElement()).resolveNames(cu) ;
	}

	void implTypeCheck(Env env, ModuleType currentModule, CompilationUnit cu) throws JamException {
		block.typeCheck(env, currentModule, cu) ;
		for(Enumeration e = catches.elements() ; e.hasMoreElements() ;) 
			((CatchClause)e.nextElement()).typeCheck(env, currentModule, cu) ;
		if (finallyBlock!=null)
			finallyBlock.typeCheck(env, currentModule, cu) ;
	}

}
