//
// Project : Jam
// File    : Type.java
// Author  : Giovanni Lagorio (gio@libertyline.com)
//

package jam ;

import java.util.* ;

abstract class Type {

	boolean isSubtypeOf(Type t) {
		return t.doubleDispatch(this) ;
	}
	protected abstract boolean doubleDispatch(Type that) ;

	boolean isSubtypeOf(VoidType vt) { return false ; }
	boolean isSubtypeOf(PrimitiveType pt) { return false ; }
	boolean isSubtypeOf(ClassType ct) { return false ; }
	boolean isSubtypeOf(InterfaceType it) { return false ; }
	boolean isSubtypeOf(MixinType mt) { return false ; }
	boolean isSubtypeOf(ArrayType mt) { return false ; }
	boolean isSubtypeOf(NilType nt) { return false ; }

	Type addDimensions(int dimensions) {
		if (dimensions==0) return this ;
		return new ArrayType(addDimensions(dimensions-1)) ;
	}

	public abstract String toString() ;

	public boolean equals(Object o) {
		return toString().equals(o.toString()) ;
	}

	public int hashCode() {
		return toString().hashCode() ;
	}

	abstract void resolveNames(CompilationUnit cu) throws JamException ;

	private static Hashtable knownTypes = new Hashtable() ;

	static void newType(String name, Type module) throws JamException {
		if (knownTypes.get(name)!=null)
			throw new JamException("type "+name+" already defined.") ;
		knownTypes.put(name, module) ;
	}

	static {
		Type t = PrimitiveType.intInstance ; // force PrimitiveType class loading
	}

	static Type getType(Type t) throws JamException {
		if (t==null) return null ;
		if (t instanceof TypeName)
			return getType( ((TypeName)t).getName().toString() ) ;
		return t ;
	}

	static Type getType(Class c) throws JamException {
		if ( c.isArray() ) {
			return new ArrayType( getType(c.getComponentType()) ) ;
		} else	return getType(c.getName()) ;
	}

	static Type getType(String name) throws JamException {
		Type t = (Type)knownTypes.get(name) ;
		if (t!=null) return t ;
		try	{
			Class c = Class.forName(name) ;
			t = c.isInterface() ? (Type)new InterfaceType(c) : new ClassType(c) ;
			knownTypes.put(name, t) ;
			return t ;
		} catch (NoClassDefFoundError ncdfe) {
			throw new ClassNotFound("Can't find module \""+name+"\".") ;
		} catch (ClassNotFoundException cnfe) {
			throw new ClassNotFound("Can't find module \""+name+"\".") ;
		} catch (UnsatisfiedLinkError ule) {
			return getType("java.lang.Object") ; // TO DO!!!
		}
	}

	static Type unaryNumericPromotion(Type t) {
		if (t==PrimitiveType.byteInstance || t==PrimitiveType.shortInstance ||
										t==PrimitiveType.charInstance)
			return PrimitiveType.intInstance ;
		return t ;
	}

	static Type binaryNumericPromotion(Type t1, Type t2) {
		if (t1==PrimitiveType.doubleInstance || t2==PrimitiveType.doubleInstance)
			return PrimitiveType.doubleInstance ;
		if (t1==PrimitiveType.floatInstance || t2==PrimitiveType.floatInstance)
			return PrimitiveType.floatInstance ;
		if (t1==PrimitiveType.longInstance || t2==PrimitiveType.longInstance)
			return PrimitiveType.longInstance ;
		return PrimitiveType.intInstance ;
	}

}
