//
// Project : Jam
// File    : TypeDeclaration.java
// Author  : Giovanni Lagorio (gio@libertyline.com)
//

package jam ;

import java.util.* ;
import invisible.jacc.parse.Token ;

abstract class TypeDeclaration extends Decl {

	protected String identifier ;
	protected Vector memberDecls ;
	protected Name fullName ;

	TypeDeclaration(Token sourcePosition_, String identifier_, Vector memberDecls_) {
		super(sourcePosition_) ;
		identifier = identifier_ ;
		memberDecls = memberDecls_ ;
	}

	protected Constructor [] constructors(ModuleType moduleType) {
		Vector cv = filterByType(memberDecls, ConstructorDeclaration.class, true) ;
		int n = cv.size() ;
		if ((this instanceof ClassDeclaration) && n==0) {
			Constructor [] constructorsArray = new Constructor[1] ;
			constructorsArray[0] = new Constructor(moduleType, new Modifiers(Modifiers.PUBLIC),
								identifier, new Type[0], new Name[0]) ;
			return constructorsArray ;
		}
		Constructor [] constructorsArray = new Constructor[n] ;
		Enumeration e = cv.elements() ;
		for(int a=0; a<n; ++a) {
			ConstructorDeclaration cd = (ConstructorDeclaration)e.nextElement() ;
			Modifiers modifiers = cd.getModifiers() ;
			String identifier = cd.getIdentifier() ;
			Vector formalParameters = cd.getFormalParameters() ;
			int nFp = formalParameters.size() ;
			Type [] arguments = new Type[nFp] ;
			for(int i=0; i<nFp; ++i)
				arguments[i] = ((FormalParameter)formalParameters.elementAt(i)).getType() ;
			Vector exceptionsVector = cd.getExceptions() ;
			int nExc = exceptionsVector.size() ;
			Name [] exceptions = new Name[nExc] ;
			copyArray(exceptionsVector, exceptions) ;
			constructorsArray[a] = new Constructor(moduleType, modifiers, identifier, arguments, exceptions) ;
		}
		return constructorsArray ;
	}

	protected Field [] fields(ModuleType moduleType) {
		Vector fdv = filterByType(memberDecls, FieldDeclaration.class, true) ;
		int n = fdv.size() ;
		int nFields = 0 ;
		for(Enumeration e=fdv.elements(); e.hasMoreElements() ;)
			nFields += ((FieldDeclaration)e.nextElement()).getDeclarators().size() ;
		Field [] fieldArray = new Field[nFields] ;
		for(int a=0, i=0; a<n; ++a) {
			FieldDeclaration fd = (FieldDeclaration)fdv.elementAt(a) ;
			Modifiers modifiers = fd.getModifiers() ;
			Type type = fd.getType() ;
			for(Enumeration e=fd.getDeclarators().elements(); e.hasMoreElements() ;) {
				VariableDeclarator vd = (VariableDeclarator)e.nextElement() ;
				VariableDeclaratorId vdid = vd.getDeclaratorId() ;
				String identifier = vdid.getIdentifier() ;
				int dims = vdid.getDimensions() ;
				Type realType = type.addDimensions(dims) ;
				fieldArray[i++] = new Field(moduleType, modifiers, realType, identifier) ;
			}
		}
		return fieldArray ;
	}

	protected Method [] methods(ModuleType moduleType) {
		Vector mv = filterByType(memberDecls, MethodDeclaration.class, true) ;
		int n = mv.size() ;
		Method [] methodsArray = new Method[n] ;
		Enumeration e = mv.elements() ;
		for(int a=0; a<n; ++a) {
			MethodDeclaration md = (MethodDeclaration)e.nextElement() ;
			MethodHeader mh = md.getHeader() ;
			Modifiers modifiers = mh.getModifiers() ;
			Type retType = mh.getReturnType() ;
			String identifier = mh.getIdentifier() ;
			Vector formalParameters = mh.getFormalParameters() ;
			int nFp = formalParameters.size() ;
			Type [] arguments = new Type[nFp] ;
			for(int i=0; i<nFp; ++i)
				arguments[i] = ((FormalParameter)formalParameters.elementAt(i)).getType() ;
			Vector exceptionsVector = mh.getExceptions() ;
			int nExc = exceptionsVector.size() ;
			Name [] exceptions = new Name[nExc] ;
			copyArray(exceptionsVector, exceptions) ;
			methodsArray[a] = new Method(moduleType, modifiers, identifier, arguments,
									exceptions, retType, md) ;
		}
		return methodsArray ;
	}

	void createModuleObject(CompilationUnit cu) throws JamException {
		fullName = cu.getPackageName().appendComponent(identifier) ;
	}

	abstract void extractModuleInformation(CompilationUnit cu) throws JamException ;

	void typeCheck(CompilationUnit cu) throws JamException {
		for(Enumeration e=memberDecls.elements(); e.hasMoreElements() ;) {
			MemberDeclaration mb = (MemberDeclaration)e.nextElement() ;
			mb.typeCheck(new Env(), (ModuleType)Type.getType(fullName.toString()), cu) ;
		}
	}

}
