//
// Project : Jam
// File    : UnaryOperator.java
// Author  : Giovanni Lagorio (gio@libertyline.com)
//

package jam ;

class UnaryOperator extends Expr {

	static final int NOT		= 0 ;
	static final int COMPLEMENT	= 1 ;
	static final int INCREMENT	= 2 ;
	static final int DECREMENT	= 3 ;
	static final int PLUS		= 4 ;
	static final int MINUS		= 5 ;

	private static final String [] operators = {
		"!", "~", "++", "--", "+", "-"
	} ;

	protected int op ;
	protected boolean isPrefixed ;
	protected Expr expr ;

	public String toString() {
		return isPrefixed ? operators[op]+expr : expr+operators[op] ;
	}

	UnaryOperator(int op_, boolean isPrefixed_, Expr expr_) {
		if (op_<0 || op_>MINUS)
			throw new RuntimeException("Internal compiler error (UO op="+op_+")") ;
		op = op_ ;
		isPrefixed = isPrefixed_ ;
		expr = expr_ ;
	}

	void resolveNames(CompilationUnit cu) throws JamException {
		expr.resolveNames(cu) ;
	}

	Expr typeCheck(Env env, ModuleType currentModule, CompilationUnit cu) throws JamException {
		expr = expr.typeCheck(env, currentModule, cu) ;
		if (op==INCREMENT || op==DECREMENT) {
			if (expr instanceof MixinFieldAccess) {
				MixinFieldAccess mfa = (MixinFieldAccess) expr ;
				Expr mixinExpr = mfa.getExpr() ;
				String fieldId = mfa.getIdentifier() ;
				MixinUnaryOperator muo = new MixinUnaryOperator(op, isPrefixed, mixinExpr, fieldId) ;
				muo.type = expr.getType() ;
				return muo ;
			}
			type = expr.getType() ;
			return this ;
		}
		if (op==PLUS || op==MINUS || op==COMPLEMENT) {
			type = Type.unaryNumericPromotion(expr.getType()) ;
			return this ;
		}
		type = PrimitiveType.booleanInstance ;
		return this ;
	}

}

